/******************************************************************************
*
* project name:    TI-Puzzlize
* file name:       load.c
* initial date:    17/11/2000
* authors:         thomas.nussbaumer@gmx.net
*
* description:     routine to handle loading of pics
*
*
* NOTE:  No animals or trees were harmed for the stones of this game
*
*
* $Id: load.c,v 1.2 2002/11/14 14:36:56 nussbaumer Exp $
*
******************************************************************************/

#include "tt095/unpack.c"
#include "tt095/ttunpack.h"    // errorcodes definition
#include "tt095/packhead.h"    // compressed header definition
#include "tt095/ttarchive.h"   // ttarchive structures and macros



SYM_ENTRY* arc_symptrs[10] = {NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
short      arc_entries[10] = {0,0,0,0,0,0,0,0,0,0};

short      active_arc      = -1;
short      active_entry    = -1;


//=============================================================================
// fetch the active_entry picture of archive active_arc
// returns 1 if succeeded
//
// NOTE: buffer must be of size (160*96/8)*2 = 1920*2 = 3840 bytes
//       at offset 1920 the second plane starts !!
//=============================================================================
short GetPicture(unsigned char* buffer) {
    SYM_ENTRY*     archive;
    unsigned short entries;
    TTAEntry*      tmp;
    unsigned short i;
    short          valid;
    unsigned char* input;
    unsigned char* src;

    if (active_arc == -1) return 0;
    archive = arc_symptrs[active_arc];

    src  = HLock(archive->handle);
    src += 2;

    entries = GetNrEntries(src);
    tmp     = GetEntryInfo(src,0);

    valid = 0;

    for (i=0;i<entries;i++,tmp++) {
        short retval;

        // check size stored in misc fields
        if (tmp->misc1[0] != 0 || tmp->misc1[1] != 160) continue;
        if (tmp->misc2[0] != 0 || tmp->misc2[1] != 96)  continue;

        if (valid==active_entry) {
            input  = GetEntryStart(src,tmp);
            retval = UnPack(input,buffer);

            if (!retval) {
                HeapUnlock(archive->handle);
                return 1;
            }
            else {
                HeapUnlock(archive->handle);
                return 0;
            }
        }
        valid++;
    }

    HeapUnlock(archive->handle);
    return 0;
}


//==============================================================================
// setups the following globals:
//
// arc_symptrs[10]
// arc_entries[10] ... number of valid entries
//==============================================================================
void FindAllArchives() {
    unsigned char *src;
    char           tmpstr[100];
    short          loop;
    unsigned short entries;
    TTAEntry*      tmp;
    unsigned short i;

    char* fname = "puzpics";

    for (loop=0;loop<10;loop++) {
        tmpstr[0] = 0;
        //sprintf(tmpstr+1,"%s%d\0",fname,loop);
        sprintf(tmpstr+1,"%s%d",fname,loop);

        arc_entries[loop] = 0;

        if (!(arc_symptrs[loop] = DerefSym(SymFind(tmpstr+9)))) {
            //printf("%s not found\n",tmpstr+1);
            //ngetchx();
            continue;
        }

        src = HLock(arc_symptrs[loop]->handle);
        src+=2;

        if (!IsTTArchive(src)) {
            HeapUnlock(arc_symptrs[loop]->handle);
            arc_symptrs[loop] = NULL;
            //printf("%s not an archive\n",tmpstr+1);
            //ngetchx();
            continue;
        }

        entries = GetNrEntries(src);
        tmp     = GetEntryInfo(src,0);

        for (i=0;i<entries;i++,tmp++) {
            // check size stored in misc fields
            if (tmp->misc1[0] != 0 || tmp->misc1[1] != 160) {
                //printf("%s wrong misc1\n",tmpstr+1);
                //ngetchx();
                continue;
            }
            if (tmp->misc2[0] != 0 || tmp->misc2[1] != 96) {
                //printf("%s wrong misc2\n",tmpstr+1);
                //ngetchx();
                continue;
            }

            arc_entries[loop]++;
        }

        HeapUnlock(arc_symptrs[loop]->handle);

        if (!arc_entries[loop]) {
            arc_symptrs[loop] = NULL;
                //printf("%s - no entries\n",tmpstr+1);
                //ngetchx();
        }
    }

    active_arc   = -1;
    active_entry = -1;

    for (i=0;i<10;i++) {
        if (arc_symptrs[i]) {
            active_arc   = i;
            active_entry = 0;
            break;
        }
    }
}


//=============================================================================
// set next picture active
//=============================================================================
void NextPicture() {
    short i;
    if (active_arc == -1) return;

    if (active_entry < arc_entries[active_arc]-1) {
        active_entry++;
    }
    else {
        active_entry = 0;
        i = active_arc;

        do {
            active_arc++;
            if (active_arc==10) active_arc = 0;
            if (arc_symptrs[active_arc]) break;
        }
        while (i != active_arc);
    }

    //for (i=0;i<2;i++) {
    //    SetPlane(i);
    //    MoveTo(0,0);
    //    printf("NOW: %d / %d",active_arc,active_entry);
    //}
    //ngetchx();

}


//=============================================================================
// set next picture active
//=============================================================================
void PreviousPicture() {
    short i;
    if (active_arc == -1) return;

    if (active_entry > 0) {
        active_entry--;
    }
    else {
        i = active_arc;

        do {
            active_arc--;
            if (active_arc==-1) active_arc = 9;
            if (arc_symptrs[active_arc]) break;
        }
        while (i != active_arc);
        active_entry = arc_entries[active_arc]-1;
    }

    //for (i=0;i<2;i++) {
    //    SetPlane(i);
    //    MoveTo(0,0);
    //    printf("NOW: %d / %d",active_arc,active_entry);
    //}
    //ngetchx();
}


//=============================================================================
// Revision History
//=============================================================================
//
// $Log: load.c,v $
// Revision 1.2  2002/11/14 14:36:56  nussbaumer
// changes for AMS2.08/V200/TIGCC 0.94
//
// Revision 1.1  2000/12/19 10:56:02  Thomas Nussbaumer
// initial version
//
//
